// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

// open a namespace
open System

// assign values
let a = 5

// typed format string
printfn "%d" a

// mutable values
let mutable b = 10
b <- 42

printfn "%d" b

// alternatively, ref cells
let c = ref 10
c := 15

printfn "%d" !c

// tuples
let d = (10, 20)
// decompose using a pattern
let part1, part2 = d
printfn "part1=%d, part2=%d" part1 part2

// Option types
let checkOptionVal(o: 'a option) = 
    if o.IsSome then
        printfn "val is Some(%A)" o.Value
    else
        printfn "val is None"

let optionVal = Some(5)
let optionVal2 = None

checkOptionVal(optionVal)
checkOptionVal(optionVal2)

let people = 
    [
        ("Willy", Some("Mercedes"));
        ("Anna", Some("Honda"));
        ("Pete", None)
    ]

let showCars(list) = 
    for (name, optionalcar) in list do
        match optionalcar with 
        | Some(car) -> printfn "%s drives a %s" name car
        | None -> printfn "%s doesn't have a car" name

showCars people


// functions
let add x y = x + y

// the type of this function is int -> int -> int
// functions are in curried format by default

printfn "%d" (add 10 20)

// lambda format functions
let add2 = fun x y -> x + y

// add2 has the same type as add now
printfn "%d" (add2 10 20)

// partial application
let add5 = add 5
printfn "%d" (add5 37)

// "chaining" calls with the pipe operator
let mult x y = x * y

printfn "%d" (mult 10 20 |> add5)

// composing functions

let mult10add5 = mult 10 >> add5

printfn "%d" (mult10add5 3)


// a list comprehension to create a sequence
let list = [10;12;17;37]

// function, inner function and lambda expression
let findEvens list = 
    let isEven x = x % 2 = 0
    Seq.filter (fun x -> isEven x) list

// this function is generic automatically -- automatic generalization
let outputList list = 
    for x in list do printf "%A " x
    printf "\n"

outputList (findEvens list)
outputList ["Hi";"there"]
outputList { 1 .. 2 .. 50 }
outputList (seq { for x in 0 .. 20 -> (x, x*x) })

// recursive functions and match expressions

let rec fact x = if x = 0 then 1 else x * fact (x - 1)

let rec altfact x = 
    match x with
    | 0I -> 1I
    | _ -> x * altfact (x - 1I)

printfn "%d" (fact 5)
printfn "%A" (altfact 100I)

// call a .NET framework function
Console.ReadLine() |> ignore
